﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Drawing;
using HeuristicLab.Common;
using HeuristicLab.Common.Resources;
using HeuristicLab.Core;
using HeuristicLab.Encodings.BinaryVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Analysis.FitnessLandscape.DistanceCalculators {

  [Item("BinaryVectorDistanceCalculator", "Calculates the hamming distance of two binary vectors")]
  [StorableClass]
  public class BinaryVectorDistanceCalculator : NamedItem, IItemDistanceCalculator {

    #region Properties

    public override bool CanChangeName { get { return false; } }
    public override bool CanChangeDescription { get { return false; } }
    public static new Image StaticItemImage { get { return VSImageLibrary.Function; } }

    #endregion

    #region Construction & Cloning

    [StorableConstructor]
    protected BinaryVectorDistanceCalculator(bool deserializing) : base(deserializing) { }

    protected BinaryVectorDistanceCalculator(BinaryVectorDistanceCalculator original, Cloner cloner)
      : base(original, cloner) {
    }

    public BinaryVectorDistanceCalculator() {
      name = ItemName;
      description = ItemDescription;
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new BinaryVectorDistanceCalculator(this, cloner);
    }

    #endregion

    #region IItemDistanceCalculator Members

    public Type ItemType {
      get { return typeof(BinaryVector); }
    }

    public double Distance(IItem x, IItem y) {
      BinaryVector a = (BinaryVector)x;
      BinaryVector b = (BinaryVector)y;
      if (a.Length != b.Length)
        throw new InvalidOperationException("Cannot compare vectors of different lengths");
      double nEqualBits = 0;
      for (int i = 0; i < a.Length; i++) {
        if (a[i] == b[i])
          nEqualBits++;
      }
      return Math.Max(a.Length, b.Length) - nEqualBits;
    }

    #endregion
  }
}
